@@ -8,17 +8,16 @@ use Carp;
 use Fcntl ':flock';
 use System::Daemon::Utils;
 
-our $VERSION = 0.13;
+use constant NEW_SUFFIX => ".new";
+
+our $VERSION = 0.14;
 our $AUTHOR = 'justnoxx';
 our $ABSTRACT = "Swiss-knife for daemonization";
 
 our $DEBUG = 0;
 
-our $LOCK;
-
 sub new {
     my ($class, %params) = @_;
-
     my $self = {};
     $self->{daemon_data}->{daemonize} = 1;
 
@@ -31,7 +30,11 @@ sub new {
     }
     
     if ($params{pidfile}) {
-        $self->{daemon_data}->{pidfile} = $params{pidfile};
+        $self->{pidfile} = $params{pidfile};
+    }
+
+    if ($params{new}) {
+        $self->{new} = 1;
     }
 
     if ($params{mkdir}) {
@@ -54,6 +57,13 @@ sub new {
     return $self;
 }
 
+sub pidfile {
+    my $self = shift;
+
+    return unless $self->{pidfile};
+
+    $self->{new} ? $self->{pidfile}.NEW_SUFFIX : $self->{pidfile};
+}
 
 sub daemonize {
     my $self = shift;
@@ -62,7 +72,7 @@ sub daemonize {
         carp "Daemonization disabled";
         return 1;
     }
-    
+
     my $dd = $self->{daemon_data};
 
     my $process_object = System::Daemon::Utils::process_object();
@@ -80,23 +90,24 @@ sub daemonize {
         };
     }
 
-    if ($dd->{pidfile}) {
-        System::Daemon::Utils::validate_pid_path($dd->{pidfile}, $dd->{mkdir});
+    if ($self->pidfile) {
+        System::Daemon::Utils::validate_pid_path($self->pidfile, $dd->{mkdir});
     }
-    System::Daemon::Utils::make_sandbox($dd) if $dd->{mkdir};
+    System::Daemon::Utils::make_sandbox($self->pidfile, $dd) if $dd->{mkdir};
     # daemon context
-    if ($dd->{pidfile}) {
+    if ($self->pidfile) {
         croak "Can't overwrite pid file of my alive instance" unless $self->ok_pid();
-        if ($dd->{pidfile}) {
-            open $LOCK, $dd->{pidfile};
+        if ($self->pidfile) {
+            open my $LOCK, $self->pidfile;
             my $got_lock = flock($LOCK, LOCK_EX | LOCK_NB);
+            $self->{_lock} = $LOCK;
             unless ($got_lock) {
-                warn "Can't get lock\n";
+                warn "Can't get lock ".$self->pidfile."\n";
                 exit 1;
             }
         }
-
-        System::Daemon::Utils::write_pid($dd->{pidfile}, undef,
+        $self->{original_pid} = $$;
+        System::Daemon::Utils::write_pid($self->pidfile, undef,
             user    =>  $dd->{user},
             group   =>  $dd->{group}
         );
@@ -138,11 +149,11 @@ sub exit {
 sub ok_pid {
     my ($self, $pidfile) = @_;
 
-    $pidfile ||= $self->{daemon_data}->{pidfile};
+    $pidfile ||= $self->pidfile;
 
     return 1 unless $pidfile;
 
-    unless (System::Daemon::Utils::pid_init($self->{daemon_data}->{pidfile})) {
+    unless (System::Daemon::Utils::pid_init($self->pidfile)) {
         croak "Can't init pidfile";
     }
 
@@ -165,10 +176,18 @@ sub cleanup {
 sub finish {
     my ($self) = @_;
 
-    my $dd = $self->{daemon_data};
+    my $pid = System::Daemon::Utils::read_pid($self->pidfile); # pid missing OR changed
+    if ($pid ne $self->{original_pid}) {
+        undef $self->{new};
+    }
 
-    if ($dd->{pidfile}) {
-        System::Daemon::Utils::delete_pidfile($dd->{pidfile});
+    if ($self->pidfile) {
+        if (-e $self->pidfile.NEW_SUFFIX) {
+            rename $self->pidfile.NEW_SUFFIX, $self->pidfile or confess "rename ".$self->pidfile.NEW_SUFFIX.", ".$self->pidfile;
+        }
+        else {
+            System::Daemon::Utils::delete_pidfile($self->pidfile);
+        }
     }
 }
 
@@ -226,7 +245,10 @@ Constructor, returns System::Daemon object. Available parameters:
     mkdir           =>  tries to create directory for pid files,
     daemonize       =>  if not true, will not daemonize, for debug reasons,
     procname        =>  after daemonize $0 will be updated to desired name,
-
+    new             =>  Write pid to newpidfile=pidfile.".new", if newpidfile
+                        dissapears or changed during daemon life, switch back to pidfile
+                        Note that even without this option, if pidfile.".new" found during destruction
+                        pidfile.".new" moved to pidfile location. These tools are for grace restart.
 
 =item daemonize
 
